use num_bigint::BigInt;
use num_traits::{One, Zero};

// Legendre symbol, returns 1, 0, or -1 mod p
fn ls(a: &BigInt, p: &BigInt) -> BigInt {
    let exp = (p - BigInt::one()) / BigInt::from(2);
    a.modpow(&exp, p)
}

// Tonelli-Shanks algorithm
pub fn mod_sqrt(n: &BigInt, p: &BigInt) -> Option<BigInt> {
    if !ls(n, p).is_one() {
        return None;
    }

    let mut q = p - 1;
    let mut s = BigInt::zero();
    while (&q & &BigInt::one()).is_zero() {
        s += 1;
        q >>= 1
    }

    if s.is_one() {
        let exp = (p + 1) / 4;
        let r1 = n.modpow(&exp, p);
        return Some(p - &r1);
    }

    let mut z = BigInt::from(2);
    while ls(&z, p) != p - 1 {
        z += 1
    }
    let mut c = z.modpow(&q, p);

    let mut r = n.modpow(&((&q + 1) / 2), p);
    let mut t = n.modpow(&q, p);
    let mut m = s;

    loop {
        if t.is_one() {
            return Some(p - &r);
        }

        let mut i = BigInt::zero();
        let mut z = t.clone();
        let mut b = c.clone();
        while !z.is_one() && i < &m - 1 {
            z = &z * &z % p;
            i += 1;
        }
        let mut e = &m - &i - 1;
        while e > BigInt::zero() {
            b = &b * &b % p;
            e -= 1;
        }
        r = &r * &b % p;
        c = &b * &b % p;
        t = &t * &c % p;
        m = i;
    }
}
